searchData={"items":[{"type":"module","title":"Exneus","doc":"An incredibly flexible and performant JSON parser, generator and formatter for Elixir.","ref":"Exneus.html"},{"type":"function","title":"Exneus.decode!/2","doc":"Decodes a binary JSON into a term.","ref":"Exneus.html#decode!/2"},{"type":"function","title":"Example - Exneus.decode!/2","doc":"iex> Exneus.decode!(\"\\\"foo\\\"\")\n    \"foo\"","ref":"Exneus.html#decode!/2-example"},{"type":"function","title":"Option details - Exneus.decode!/2","doc":"- `codecs` - Transforms a JSON binary value into an Erlang term.\n  By returning `:next`, the next codec will be called, or by returning\n  `{:halt, term :: term()}`, the term is returned as the value.\n\n  You can use the built-in codecs or your own.\n  Please see the `t::euneus_decoder.codec/0` type for details.\n\n  Default is `[]`.\n\n  Built-in codecs:\n\n  - `timestamp` - Transforms an ISO 8601 string with milliseconds into\n    an `t::erlang.timestamp/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"1970-01-01T00:00:00.000Z\\\"\", %{codecs: [:timestamp]})\n        {0, 0, 0}\n\n  - `datetime` - Transforms an ISO 8601 string into a `t::calendar.datetime/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"1970-01-01T00:00:00Z\\\"\", %{codecs: [:datetime]})\n        {{1970, 01, 01},{00, 00, 00}}\n\n  - `ipv4` - Transforms a JSON string into an `t::inet.ip4_address/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"127.0.0.1\\\"\", %{codecs: [:ipv4]})\n        {127, 0, 0, 1}\n\n  - `ipv6` - Transforms a JSON string into an `t::inet.ip6_address/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"::\\\"\", %{codecs: [:ipv6]})\n        {0, 0, 0, 0, 0, 0, 0, 0}\n        iex> Exneus.decode!(\"\\\"::1\\\"\", %{codecs: [:ipv6]})\n        {0, 0, 0, 0, 0, 0, 0, 1}\n        iex> Exneus.decode!(\"\\\"::192.168.42.2\\\"\", %{codecs: [:ipv6]})\n        {0, 0, 0, 0, 0, 0, 49320, 10754}\n        iex> Exneus.decode!(\"\\\"::ffff:192.168.42.2\\\"\", %{codecs: [:ipv6]})\n        {0, 0, 0, 0, 0, 65535, 49320, 10754}\n        iex> Exneus.decode!(\"\\\"3ffe:b80:1f8d:2:204:acff:fe17:bf38\\\"\", %{codecs: [:ipv6]})\n        {16382, 2944, 8077, 2, 516, 44287, 65047, 48952}\n        iex> Exneus.decode!(\"\\\"fe80::204:acff:fe17:bf38\\\"\", %{codecs: [:ipv6]})\n        {65152, 0, 0, 0, 516, 44287, 65047, 48952}\n\n  - `pid` - Transforms a JSON string into an `t::erlang.pid/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"<0.92.0>\\\"\", %{codecs: [:pid]})\n        ...> == :erlang.list_to_pid(~c\"<0.92.0>\")\n        true\n\n  - `port` - Transforms a JSON string into an `t::erlang.port/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"#Port<0.1>\\\"\", %{codecs: [:port]})\n        ...> == :erlang.list_to_port(~c\"#Port<0.1>\")\n        true\n\n  - `reference` - Transforms a JSON string into an `t::erlang.reference/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"\\\"#Ref<0.314572725.1088159747.110918>\\\"\", %{codecs: [:reference]})\n        ...> == :erlang.list_to_ref(~c\"#Ref<0.314572725.1088159747.110918>\")\n        true\n\n  Custom codec example:\n\n      iex> Exneus.decode!(\"\\\"foo\\\"\", %{codecs: [fn (\"foo\") -> {:halt, :foo} end]})\n      :foo\n\n- `null` - Defines which term should be considered null.\n\n  Default is `nil`.\n\n  _Example:_\n\n      iex> Exneus.decode!(\"null\", %{null: :null})\n      :null\n\n- `binary_to_float` - Overrides the default binary to float conversion.\n\n- `binary_to_integer` - Overrides the default binary to integer conversion..\n\n- `array_start` - Overrides the `t::json.array_start_fun/0` callback.\n\n- `array_push` - Overrides the `t::json.array_push_fun/0` callback.\n\n- `array_finish` - Overrides the `t::json.array_finish_fun/0` callback.\n\n  In addition to the custom function, there are:\n\n  - `ordered` - Returns the array in the same order as the JSON.\n\n    That's the slower option.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"[1,2,3]\", %{array_finish: :ordered})\n        [1,2,3]\n\n  - `reversed` - Returns the array in a reversed order.\n\n    That's the faster option.\n\n    _Example:_\n\n        iex> Exneus.decode!(\"[1,2,3]\", %{array_finish: :reversed})\n        [3,2,1]\n\n  Default is `ordered`.\n\n- `object_start` - Overrides the `t::json.object_start_fun/0` callback.\n\n- `object_keys` - Transforms JSON objects key into Erlang term.\n\n  In addition to the custom function, there are:\n\n  - `binary` - Returns the key as `t::erlang.binary/0`.\n  \n  - `copy` - Copies the key via `:binary.copy/1` returning it as `t::erlang.binary/0`.\n\n  - `atom` - Returns the key as `t::erlang.atom/0` via `:erlang.binary_to_atom/2`.\n\n  - `existing_atom` - Returns the key as `t::erlang.atom/0` via\n    `:erlang.binary_to_existing_atom/2`.\n\n  Default is `binary`.\n\n- `object_push` - Overrides the `t::json.object_push_fun/0` callback.\n\n- `object_finish` - Overrides the `t::json.object_finish_fun/0` callback.\n\n  In addition to the custom function, there are:\n\n  - `map` - Returns the object as a `t::erlang.map/0`.\n\n    That's the slower option.\n\n    _Example:_\n\n        iex> Exneus.decode!(\n        ...>   \"{\\\"a\\\":\\\"a\\\",\\\"b\\\":\\\"b\\\",\\\"c\\\":\\\"c\\\"}\",\n        ...>   %{object_finish: :map}\n        ...> )\n        %{<<\"a\">> => <<\"a\">>,<<\"b\">> => <<\"b\">>,<<\"c\">> => <<\"c\">>}\n\n  - `keyword_list` - Returns the object as an ordered `t:keyword/0`.\n\n    _Example:_\n\n        iex> Exneus.decode!(\n        ...>   \"{\\\"a\\\":\\\"a\\\",\\\"b\\\":\\\"b\\\",\\\"c\\\":\\\"c\\\"}\",\n        ...>   %{object_finish: :keyword_list}\n        ...> )\n        [{\"a\", \"a\"},{\"b\", \"b\"},{\"c\", \"c\"}]\n\n  - `reversed_keyword_list` - Returns the object as a reversed `t:keyword/0`.\n\n    That's the faster option.\n\n    _Example:_\n\n        iex> Exneus.decode!(\n        ...>   \"{\\\"a\\\":\\\"a\\\",\\\"b\\\":\\\"b\\\",\\\"c\\\":\\\"c\\\"}\",\n        ...>   %{object_finish: :reversed_keyword_list}\n        ...> )\n        [{\"c\", \"c\"},{\"b\", \"b\"},{\"a\", \"a\"}]\n\n  Default is `map`.","ref":"Exneus.html#decode!/2-option-details"},{"type":"function","title":"Exneus.decode_stream_continue!/2","doc":"Continue parsing a stream of bytes of a JSON value.","ref":"Exneus.html#decode_stream_continue!/2"},{"type":"function","title":"Example - Exneus.decode_stream_continue!/2","doc":"iex> {:continue, state} = Exneus.decode_stream_start!(\"{\\\"foo\\\":\")\n    iex> Exneus.decode_stream_continue!(\"1}\", state)\n    {:end_of_input, %{\"foo\" => 1}}","ref":"Exneus.html#decode_stream_continue!/2-example"},{"type":"function","title":"Exneus.decode_stream_end!/1","doc":"End parsing a stream of bytes of a JSON value.","ref":"Exneus.html#decode_stream_end!/1"},{"type":"function","title":"Example - Exneus.decode_stream_end!/1","doc":"iex> {:continue, state} = Exneus.decode_stream_start!(\"123\")\n    iex> Exneus.decode_stream_end!(state)\n    {:end_of_input, 123}","ref":"Exneus.html#decode_stream_end!/1-example"},{"type":"function","title":"Exneus.decode_stream_start!/2","doc":"Begin parsing a stream of bytes of a JSON value.","ref":"Exneus.html#decode_stream_start!/2"},{"type":"function","title":"Exneus.encode!/2","doc":"Encodes a term into a binary JSON.","ref":"Exneus.html#encode!/2"},{"type":"function","title":"Example - Exneus.encode!/2","doc":"iex> Exneus.encode!(:foo)\n    \"\\\"foo\\\"\"","ref":"Exneus.html#encode!/2-example"},{"type":"function","title":"Option details - Exneus.encode!/2","doc":"> #### Note {: .info}\n>\n> For better visualization and understanding, all options examples use\n> `Exneus.encode!/2`, which returns a binary.\n\n- `codecs` - Transforms tuples into any other Erlang term that will be encoded\n  again into a JSON value. By returning `:next`, the next codec will be called,\n  or by returning `{:halt, term :: term()}`, the term will be encoded again.\n\n  You can use the built-in codecs or your own.\n  Please see the `t::euneus_encoder.codec/0` type for details.\n\n  Default is `[]`.\n\n  Built-in codecs:\n\n  - `timestamp` - Transforms an `t::erlang.timestamp/0` into an ISO 8601 string\n    with milliseconds.\n\n    _Example:_\n\n        iex> Exneus.encode!({0, 0, 0}, %{codecs: [:timestamp]})\n        \"\\\"1970-01-01T00:00:00.000Z\\\"\"\n\n  - `datetime` - Transforms a `t::calendar.datetime/0` into an ISO 8601 string.\n\n    _Example:_\n\n        iex> Exneus.encode!({{1970, 01, 01}, {00, 00, 00}}, %{codecs: [:datetime]})\n        \"\\\"1970-01-01T00:00:00Z\\\"\"\n\n  - `ipv4` - Transforms an `t::inet.ip4_address/0` into a JSON string.\n\n    _Example:_\n\n        iex> Exneus.encode!({127, 0, 0, 1}, %{codecs: [:ipv4]})\n        \"\\\"127.0.0.1\\\"\"\n\n  - `ipv6` - Transforms an `t::inet.ip6_address/0` into a JSON string.\n\n    _Example:_\n\n        iex> Exneus.encode!({0, 0, 0, 0, 0, 0, 0, 0}, %{codecs: [:ipv6]})\n        \"\\\"::\\\"\"\n        iex> Exneus.encode!({0, 0, 0, 0, 0, 0, 0, 1}, %{codecs: [:ipv6]})\n        \"\\\"::1\\\"\"\n        iex> Exneus.encode!({0, 0, 0, 0, 0, 0, 49320, 10754}, %{codecs: [:ipv6]})\n        \"\\\"::192.168.42.2\\\"\"\n        iex> Exneus.encode!({0, 0, 0, 0, 0, 65535, 49320, 10754}, %{codecs: [:ipv6]})\n        \"\\\"::ffff:192.168.42.2\\\"\"\n        iex> Exneus.encode!({16382, 2944, 8077, 2, 516, 44287, 65047, 48952}, %{codecs: [:ipv6]})\n        \"\\\"3ffe:b80:1f8d:2:204:acff:fe17:bf38\\\"\"\n        iex> Exneus.encode!({65152, 0, 0, 0, 516, 44287, 65047, 48952}, %{codecs: [:ipv6]})\n        \"\\\"fe80::204:acff:fe17:bf38\\\"\"\n\n  - `records` - Transforms records into JSON objects.\n\n    _Example:_\n\n        iex> Exneus.encode!(\n        ...>   # Same as `Record.defrecord(:foo, :bar, :baz)`\n        ...>   {:foo, :bar, :baz},\n        ...>   %{codecs: [{:records, %{\n        ...>       # Use `Record.extract/2` to extract those record informations\n        ...>       foo: {[:bar, :baz], 3}\n        ...>   }}]}\n        ...> )\n        if String.to_integer(System.otp_release) >= 26 do\n          \"{\\\"bar\\\":\\\"bar\\\",\\\"baz\\\":\\\"baz\\\"}\"\n        else\n          \"{\\\"baz\\\":\\\"baz\\\",\\\"bar\\\":\\\"bar\\\"}\"\n        end\n\n  Custom codec example:\n\n      iex> Exneus.encode!({:foo}, %{codecs: [fn ({:foo}) -> {:halt, :foo} end]})\n      \"\\\"foo\\\"\"\n\n- `codec_callback` - Overrides the default codec resolver.\n\n  Default is `euneus_encoder.codec_callback/2`.\n\n- `nulls` - Defines which values should be encoded as null.\n\n  Default is `[nil]`.\n\n  _Example:_\n\n      iex> Exneus.encode!([:null, nil, :foo], %{nulls: [:null, nil]})\n      \"[null,null,\\\"foo\\\"]\"\n\n- `skip_values` - Defines which map values should be ignored.\n  This option permits achieves the same behavior as Javascript,\n  which ignores undefined values of objects.\n\n  Default is `[]`.\n\n  _Example:_\n\n      iex> Exneus.encode!(\n      ...>   %{foo: :bar, bar: :undefined},\n      ...>   %{skip_values: [:undefined]}\n      ...> )\n      \"{\\\"foo\\\":\\\"bar\\\"}\"\n\n- `key_to_binary` - Overrides the default conversion of map keys to a string.\n\n  Default is `:euneus_encoder.key_to_binary/1` .\n\n- `sort_keys` - Defines if the object keys should be sorted.\n\n  Default is `false`.\n\n  _Example:_\n\n      iex> Exneus.encode!(%{c: :c, a: :a, b: :b}, %{sort_keys: true})\n      \"{\\\"a\\\":\\\"a\\\",\\\"b\\\":\\\"b\\\",\\\"c\\\":\\\"c\\\"}\"\n\n- `keyword_lists` - If true, converts keyword_lists into objects.\n\n  Default is `false`.\n\n  _Example:_\n\n      iex> Exneus.encode!([:baz, foo: :bar], %{keyword_lists: true})\n      \"{\\\"foo\\\":\\\"bar\\\",\\\"baz\\\":true}\"\n      iex> Exneus.encode!(\n      ...>     [foo: :bar, baz: true],\n      ...>     # Overrides the default is keyword list check:\n      ...>     %{keyword_lists: {true, fn ([{_, _} | _]) -> true end}}\n      ...> )\n      \"{\\\"foo\\\":\\\"bar\\\",\\\"baz\\\":true}\"\n\n- `escape` - Overrides the default string escaping.\n\n  Default is `:euneus_encoder.escape/1`.\n\n- `encode_integer` - Overrides the default integer encoder.\n\n  Default is `:euneus_encoder.encode_integer/2`.\n\n- `encode_float` - Overrides the default float encoder.\n\n  Default is `:euneus_encoder.encode_float/2`.\n\n- `encode_atom` - Overrides the default atom encoder.\n\n  Default is `:euneus_encoder.encode_atom/2`.\n\n- `encode_list` - Overrides the default list encoder.\n\n  Default is `:euneus_encoder.encode_list/2`.\n\n- `encode_map` - Overrides the default map encoder.\n\n  Default is the private function Exneus.encode_map/2.\n\n- `encode_tuple` - Overrides the default tuple encoder.\n\n  Default is `:euneus_encoder.encode_tuple/2`, which raises\n  `:unsupported_tuple` error.\n\n- `encode_pid` - Overrides the default pid encoder.\n\n  Default is `:euneus_encoder.encode_pid/2`, which raises\n  `:unsupported_pid` error.\n\n- `encode_port` - Overrides the default port encoder.\n\n  Default is `:euneus_encoder.encode_port/2`, which raises\n  `:unsupported_port` error.\n\n- `encode_reference` - Overrides the default reference encoder.\n\n  Default is `:euneus_encoder.encode_reference/2`, which raises\n  `:unsupported_reference` error.\n\n- `encode_term` - Overrides the default encoder for unsupported terms,\n  like functions.\n\n  Default is `:euneus_encoder.encode_term/2`, which raises\n  `:unsupported_term` error.","ref":"Exneus.html#encode!/2-option-details"},{"type":"function","title":"Exneus.encode_to_iodata!/2","doc":"Encode a term into an iodata JSON.","ref":"Exneus.html#encode_to_iodata!/2"},{"type":"function","title":"Example - Exneus.encode_to_iodata!/2","doc":"iex> Exneus.encode_to_iodata!(:foo)\n    [?\", \"foo\", ?\"]","ref":"Exneus.html#encode_to_iodata!/2-example"},{"type":"function","title":"Exneus.format/2","doc":"Formats a binary JSON.","ref":"Exneus.html#format/2"},{"type":"function","title":"Example - Exneus.format/2","doc":"iex> opts = %{\n    ...>   indent_type: :tabs,\n    ...>   indent_width: 1,\n    ...>   spaced_values: true,\n    ...>   crlf: :n\n    ...> }\n    %{crlf: :n, indent_type: :tabs, indent_width: 1, spaced_values: true}\n    iex> Exneus.format(\" \\n{\\\"foo\\\"  :  [ true  , \\n null ] \\n  }  \", opts)\n    \"{\\n\\t\\\"foo\\\": [\\n\\t\\ttrue,\\n\\t\\tnull\\n\\t]\\n}\"","ref":"Exneus.html#format/2-example"},{"type":"function","title":"Option details - Exneus.format/2","doc":"> #### Note {: .info}\n>\n> There is no default for any option, all are required.\n\n- `indent_type` - Indent using `tabs` or `spaces`.\n\n  - `tabs` - The indent char will be `?\\t`.\n\n  - `spaces` - The indent char will be `?\\s`.\n\n- `indent_width` - The `indent_type` will be copied N times based on it.\n\n- `spaced_values` - Defines if keys and values of objects should be\n  spaced by one `?\\s` char.\n\n- `crlf` - Defines the Carriage Return/Line Feed.\n\n  - `r` - The CRLF will be `\"\\r\"`.\n\n  - `n` - The CRLF will be `\"\\n\"`.\n\n  - `rn` - The CRLF will be `\"\\r\\n\"`.\n\n  - `none` - The CRLF will be `\"\"`.","ref":"Exneus.html#format/2-option-details"},{"type":"function","title":"Exneus.format_to_iodata/2","doc":"Formats a binary JSON.","ref":"Exneus.html#format_to_iodata/2"},{"type":"function","title":"Exneus.minify/1","doc":"Minifies a binary JSON.","ref":"Exneus.html#minify/1"},{"type":"function","title":"Example - Exneus.minify/1","doc":"iex> Exneus.minify(\" \\n{\\\"foo\\\"  :  [ true  , \\n null ] \\n  }  \")\n    \"{\\\"foo\\\":[true,null]}\"","ref":"Exneus.html#minify/1-example"},{"type":"function","title":"Exneus.minify_to_iodata/1","doc":"Minifies a binary JSON.","ref":"Exneus.html#minify_to_iodata/1"},{"type":"type","title":"Exneus.decode_options/0","doc":"","ref":"Exneus.html#t:decode_options/0"},{"type":"type","title":"Exneus.encode_options/0","doc":"","ref":"Exneus.html#t:encode_options/0"},{"type":"type","title":"Exneus.is_keyword_list/0","doc":"","ref":"Exneus.html#t:is_keyword_list/0"},{"type":"extras","title":"Overview","doc":"# Exneus\n\n[![Github Actions](https://github.com/williamthome/exneus/workflows/CI/badge.svg)](https://github.com/williamthome/exneus/actions)\n[![Coverage](https://raw.githubusercontent.com/cicirello/jacoco-badge-generator/main/tests/100.svg)](https://github.com/williamthome/exneus/actions/workflows/ci.yml)\n[![Elixir Versions](https://img.shields.io/badge/Elixir%2F-12%2B-purple?style=flat-square)](https://elixir-lang.org/)\n[![Erlang Versions](https://img.shields.io/badge/Erlang%2FOTP-24%2B-green?style=flat-square)](http://www.erlang.org)\n[![Latest Release](https://img.shields.io/github/release/williamthome/exneus.svg?style=flat-square)](https://github.com/williamthome/exneus/releases/latest)\n[![Hex Version](https://img.shields.io/hexpm/v/exneus.svg)](https://hex.pm/packages/exneus)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/exneus/)\n[![Total Download](https://img.shields.io/hexpm/dt/exneus.svg)](https://hex.pm/packages/exneus)\n[![License](https://img.shields.io/hexpm/l/exneus.svg)](https://github.com/williamthome/exneus/blob/main/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/williamthome/exneus.svg)](https://github.com/williamthome/exneus/commits/main)\n\nAn incredibly flexible and performant JSON parser, generator and formatter for Elixir.\n\nExneus is a wrapper for Elixir of [Euneus](https://github.com/williamthome/euneus),\nan Erlang library built on the top of the new [OTP json module](https://erlang.org/documentation/doc-15.0-rc3/lib/stdlib-6.0/doc/html/json.html).\n\nBoth encoder and decoder fully conform to [RFC 8259](https://datatracker.ietf.org/doc/html/rfc8259)\nand [ECMA 404](https://ecma-international.org/publications-and-standards/standards/ecma-404/) standards\nand are tested using [JSONTestSuite](https://github.com/nst/JSONTestSuite).\n\nFurther explanation and examples are available at [hexdocs](https://hexdocs.pm/exneus).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"```elixir\ndef deps do\n  [\n    {:json_polyfill, \"~> 0.1\"}, # Required only for OTP < 27\n    {:exneus, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic usage - Overview","doc":"```elixir\niex(1)> Exneus.encode!(%{name: \"Joe Armstrong\", age: 68, nationality: \"British\"})\n\"{\\\"name\\\":\\\"Joe Armstrong\\\",\\\"age\\\":68,\\\"nationality\\\":\\\"British\\\"}\"\niex(2)> Exneus.decode!(v(1))\n%{\"age\" => 68, \"name\" => \"Joe Armstrong\", \"nationality\" => \"British\"}\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Encode - Overview","doc":"Functions that provide JSON encoding:\n\n- `Exneus.encode!/2`\n- `Exneus.encode_to_iodata!/2`","ref":"readme.html#encode"},{"type":"extras","title":"Decode - Overview","doc":"Functions that provide JSON decoding:\n\n- `Exneus.decode!/2`","ref":"readme.html#decode"},{"type":"extras","title":"Stream - Overview","doc":"Functions that provide JSON decode streaming:\n\n- `Exneus.decode_stream_start!/2`\n- `Exneus.decode_stream_continue!/2`\n- `Exneus.decode_stream_end!/1`","ref":"readme.html#stream"},{"type":"extras","title":"Format - Overview","doc":"Functions that provide JSON formatting:\n\n- `Exneus.minify/1`\n- `Exneus.minify_to_iodata/1`\n- `Exneus.format/2`\n- `Exneus.format_to_iodata/2`","ref":"readme.html#format"},{"type":"extras","title":"Benchmark - Overview","doc":"See [BENCHMARK.md](benchmark/BENCHMARK.md) for more information.","ref":"readme.html#benchmark"},{"type":"extras","title":"Encode benchmark - Overview","doc":"```console\n##### With input Blockchain #####\nName                 ips\neuneus           36.95 K\njson (OTP)       35.85 K - 1.03x slower +0.83 μs\nExneus           35.71 K - 1.03x slower +0.94 μs\njiffy            34.70 K - 1.06x slower +1.76 μs\nJason            26.75 K - 1.38x slower +10.32 μs\nthoas            16.40 K - 2.25x slower +33.92 μs\njsone            15.52 K - 2.38x slower +37.38 μs\njsx               4.21 K - 8.77x slower +210.29 μs\nJSON              3.88 K - 9.52x slower +230.47 μs\n\nMemory usage statistics:\n\nName               average\neuneus            84.20 KB\njson (OTP)        82.66 KB - 0.98x memory usage -1.53125 KB\nExneus            84.13 KB - 1.00x memory usage -0.06250 KB\njiffy              7.79 KB - 0.09x memory usage -76.40625 KB\nJason             78.32 KB - 0.93x memory usage -5.87500 KB\nthoas             89.41 KB - 1.06x memory usage +5.22 KB\njsone            178.56 KB - 2.12x memory usage +94.37 KB\njsx              397.30 KB - 4.72x memory usage +313.11 KB\nJSON             481.30 KB - 5.72x memory usage +397.11 KB\n```","ref":"readme.html#encode-benchmark"},{"type":"extras","title":"Decode benchmark - Overview","doc":"```console\n##### With input Blockchain #####\nName                 ips\njson (OTP)       18.79 K\neuneus           18.06 K - 1.04x slower +2.16 μs\nExneus           17.68 K - 1.06x slower +3.34 μs\nJason            15.16 K - 1.24x slower +12.74 μs\njsone            13.82 K - 1.36x slower +19.17 μs\njiffy            12.76 K - 1.47x slower +25.15 μs\nthoas            11.89 K - 1.58x slower +30.89 μs\njsx               4.93 K - 3.81x slower +149.64 μs\nJSON              2.43 K - 7.74x slower +358.48 μs\n\nMemory usage statistics:\n\nName               average\njson (OTP)        35.95 KB\neuneus            36.93 KB - 1.03x memory usage +0.98 KB\nExneus            37.05 KB - 1.03x memory usage +1.09 KB\nJason             51.63 KB - 1.44x memory usage +15.67 KB\njsone            131.70 KB - 3.66x memory usage +95.74 KB\njiffy              1.55 KB - 0.04x memory usage -34.40625 KB\nthoas             51.41 KB - 1.43x memory usage +15.46 KB\njsx              315.84 KB - 8.78x memory usage +279.89 KB\nJSON            1042.65 KB - 29.00x memory usage +1006.69 KB\n```","ref":"readme.html#decode-benchmark"},{"type":"extras","title":"Sponsors - Overview","doc":"If you like this tool, please consider [sponsoring me](https://github.com/sponsors/williamthome).\nI'm thankful for your never-ending support :heart:\n\nI also accept coffees :coffee:\n\n[![\"Buy Me A Coffee\"](https://www.buymeacoffee.com/assets/img/custom_images/orange_img.png)](https://www.buymeacoffee.com/williamthome)","ref":"readme.html#sponsors"},{"type":"extras","title":"License - Overview","doc":"Copyright (c) 2024 [William Fank Thomé](https://github.com/williamthome)\n\nExneus is 100% open-source and community-driven. All components are\navailable under the Apache 2 License on [GitHub](https://github.com/williamthome/exneus).\n\nSee [LICENSE.md](LICENSE.md) for more information.","ref":"readme.html#license"},{"type":"extras","title":"License","doc":"# Apache License\n\nVersion 2.0, January 2004\n\n \n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions - License","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License - License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License - License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution - License","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions - License","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks - License","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty - License","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability - License","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability - License","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2024, William Fank Thomé  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n \n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"},{"type":"extras","title":"Benchmark","doc":"# Exneus Benchmark\n\n> - Operating System: Linux\n> - CPU Information: 12th Gen Intel(R) Core(TM) i9-12900HX\n> - Number of Available Cores: 24\n> - Available memory: 31.09 GB\n> - Elixir 1.17.2\n> - Erlang 27.0.1\n> - JIT enabled: true\n>\n> Benchmark suite executing with the following configuration:\n>\n> - warmup: 1 s\n> - time: 10 s\n> - memory time: 1 s\n> - reduction time: 0 ns\n> - parallel: 1\n> - inputs: Blockchain","ref":"benchmark.html"},{"type":"extras","title":"Encode - Benchmark","doc":"Encode run time comparison:\n\n```console\n##### With input Blockchain #####\nName                 ips        average  deviation         median         99th %\neuneus           36.95 K       27.06 μs    ±34.15%       26.10 μs       34.76 μs\njson (OTP)       35.85 K       27.90 μs    ±14.90%       27.22 μs       35.61 μs\nExneus           35.71 K       28.00 μs    ±10.67%       27.18 μs       35.77 μs\njiffy            34.70 K       28.82 μs    ±15.29%       28.14 μs       37.15 μs\nJason            26.75 K       37.38 μs    ±16.27%       35.48 μs       48.25 μs\nthoas            16.40 K       60.99 μs     ±9.55%       57.84 μs       72.94 μs\njsone            15.52 K       64.44 μs    ±15.47%       63.12 μs       93.81 μs\njsx               4.21 K      237.35 μs    ±17.87%      225.05 μs      461.23 μs\nJSON              3.88 K      257.54 μs    ±13.80%      248.47 μs      403.79 μs\n\nComparison:\neuneus           36.95 K\njson (OTP)       35.85 K - 1.03x slower +0.83 μs\nExneus           35.71 K - 1.03x slower +0.94 μs\njiffy            34.70 K - 1.06x slower +1.76 μs\nJason            26.75 K - 1.38x slower +10.32 μs\nthoas            16.40 K - 2.25x slower +33.92 μs\njsone            15.52 K - 2.38x slower +37.38 μs\njsx               4.21 K - 8.77x slower +210.29 μs\nJSON              3.88 K - 9.52x slower +230.47 μs\n```\n\n![Encode run time](assets/encode_run_time.png)\n\nEncode memory usage comparison:\n\n```console\n##### With input Blockchain #####\nName          Memory usage\neuneus            84.20 KB\njson (OTP)        82.66 KB - 0.98x memory usage -1.53125 KB\nExneus            84.13 KB - 1.00x memory usage -0.06250 KB\njiffy              7.79 KB - 0.09x memory usage -76.40625 KB\nJason             78.32 KB - 0.93x memory usage -5.87500 KB\nthoas             89.41 KB - 1.06x memory usage +5.22 KB\njsone            178.56 KB - 2.12x memory usage +94.37 KB\njsx              397.30 KB - 4.72x memory usage +313.11 KB\nJSON             481.30 KB - 5.72x memory usage +397.11 KB\n```\n\n![Encode memory usage](assets/encode_memory_usage.png)","ref":"benchmark.html#encode"},{"type":"extras","title":"Decode - Benchmark","doc":"Decode run time comparison:\n\n```console\n##### With input Blockchain #####\nName                 ips        average  deviation         median         99th %\njson (OTP)       18.79 K       53.21 μs    ±16.52%       51.72 μs       61.94 μs\neuneus           18.06 K       55.37 μs     ±6.91%       54.64 μs       63.03 μs\nExneus           17.68 K       56.55 μs    ±10.11%       55.98 μs       65.02 μs\nJason            15.16 K       65.95 μs     ±9.13%       65.09 μs       82.98 μs\njsone            13.82 K       72.38 μs     ±5.79%       72.07 μs       83.88 μs\njiffy            12.76 K       78.36 μs    ±23.38%       68.45 μs      145.49 μs\nthoas            11.89 K       84.10 μs     ±5.79%       83.40 μs       96.02 μs\njsx               4.93 K      202.84 μs    ±20.64%      189.98 μs      379.74 μs\nJSON              2.43 K      411.69 μs    ±13.42%      401.45 μs      699.87 μs\n\nComparison:\njson (OTP)       18.79 K\neuneus           18.06 K - 1.04x slower +2.16 μs\nExneus           17.68 K - 1.06x slower +3.34 μs\nJason            15.16 K - 1.24x slower +12.74 μs\njsone            13.82 K - 1.36x slower +19.17 μs\njiffy            12.76 K - 1.47x slower +25.15 μs\nthoas            11.89 K - 1.58x slower +30.89 μs\njsx               4.93 K - 3.81x slower +149.64 μs\nJSON              2.43 K - 7.74x slower +358.48 μs\n```\n\n![Decode run time](assets/decode_run_time.png)\n\nDecode memory usage comparison:\n\n```console\n##### With input Blockchain #####\nName               average  deviation         median         99th %\njson (OTP)        35.95 KB     ±0.00%       35.95 KB       35.95 KB\neuneus            36.93 KB     ±0.00%       36.93 KB       36.93 KB\nExneus            37.05 KB     ±0.00%       37.05 KB       37.05 KB\nJason             51.63 KB     ±0.00%       51.63 KB       51.63 KB\njsone            131.70 KB     ±0.00%      131.70 KB      131.70 KB\njiffy              1.55 KB     ±0.00%        1.55 KB        1.55 KB\nthoas             51.41 KB     ±0.00%       51.41 KB       51.41 KB\njsx              315.84 KB     ±0.00%      315.84 KB      315.84 KB\nJSON            1042.65 KB     ±0.00%     1042.65 KB     1042.65 KB\n\nComparison:\njson (OTP)        35.95 KB\neuneus            36.93 KB - 1.03x memory usage +0.98 KB\nExneus            37.05 KB - 1.03x memory usage +1.09 KB\nJason             51.63 KB - 1.44x memory usage +15.67 KB\njsone            131.70 KB - 3.66x memory usage +95.74 KB\njiffy              1.55 KB - 0.04x memory usage -34.40625 KB\nthoas             51.41 KB - 1.43x memory usage +15.46 KB\njsx              315.84 KB - 8.78x memory usage +279.89 KB\nJSON            1042.65 KB - 29.00x memory usage +1006.69 KB\n```\n\n![Decode memory usage](assets/decode_memory_usage.png)","ref":"benchmark.html#decode"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}